function [nameList,ftrs] = getMfccFtrs(fileList)
%
% [nameList,ftrs] = getMfccFtrs(fileList)
%
% Extract MFCC features (mean and covariance) from a list of audio files
%
% Inputs:
%   fileList:   cell array of audio files
%
% Outputs:
%   nameList:   cell array of file names (with path and extension
%               stripped)
%   ftrs:       cell array containing mean (ftrs{i}.mu) and covariance
%               (ftrs{i}.sigma) of MFCC's for each file
%
% 2010/11/30 Courtenay Cotton (cvcotton@ee.columbia.edu)

nameList = cell(length(fileList),1);
ftrs = cell(length(fileList),1);

% Process each file in input list
for i = 1:length(fileList)
    
    % Input file
    fileName = fileList{i};
    [pathstr,name,ext] = fileparts(fileName);
    nameList{i} = name;
    
    % Read in data
    if( strcmp(ext,'.wav') )
        [x,fs] = wavread(fileName);
    elseif( strcmp(ext,'.mp3') )
        [x,fs] = mp3read(fileName);
    else
        disp(['Unknown file type of: ',fileName,'; continuing to next file..']);
    end
    
    % Ignore 2nd channel if not mono
    if( size(x,2) == 2 )
        x = x(:,1);
    end
    
    disp(['Getting MFCC features for: ',fileName]);
    
    % Calculate MFCC's:
    cepstra = melfcc(x,fs,'maxfreq',7000,'numcep',25)';
    
    % Make sure there are some frames
    if( ~isempty(cepstra) )
        
        % Remove NaN's
        cepstra = cepstra(~isnan(cepstra(:,1)),:);
        
        % Make sure there are STILL frames
        if( ~isempty(cepstra) )
            ftrs{i}.mu = mean(cepstra);
            ftrs{i}.sigma = cov(cepstra);
        else
            ftrs{i}.mu = [];
            ftrs{i}.sigma = [];
        end
    end
    
end

