function [] = runBaseline(distType,plotResults,dataDir)
%
% [] = runBaseline()
% [] = runBaseline(distType)
% [] = runBaseline(distType,plotResults)
% [] = runBaseline(distType,plotResults,dataDir)
%
% Run baseline audio concept classification system
%
% Inputs (optional):
%   distType:   method used to calculate distance between audio clips
%               (based on their MFCC statistics)
%               distType = 1 uses Mahalanobis distance (default)
%               distType = 2 uses KL divergence
%   plotResults:    1 to display bar graph of average precision results,
%                   0 otherwise (default)
%   dataDir:    location of audio files and labels, e.g. '/somePath/data'
%
% 2010/11/30 Courtenay Cotton (cvcotton@ee.columbia.edu)

%%%%%%%%%%%%%%%%%%%%%

% distType: 1 = Mahalanobis distance, 2 = KL divergence
if(nargin<1); distType = 1; end;

if(nargin<2); plotResults = 1; end;

% Default location of audio files in this package
if(nargin<3)
    path = fileparts(which('runBaseline'));
    dataDir = fullfile(path,'data');
end

%%%%%%%%%%%%%%%%%%%%%
% Feature extraction:

audioDir = fullfile(dataDir,'mp3');
labelDir = fullfile(dataDir,'label');

% Get list of audio files in directory:
[fileList] = getFileList(audioDir);

% Collect MFCC statistics from files:
[nameList,ftrs] = getMfccFtrs(fileList);

% Get list of concept names
CC = importdata(fullfile(labelDir,'order_labels.txt'));
nConcept = length(CC);

%%%%%%%%%%%%%%%%%%%%%
% Experiment:

% Keep track of SVM parameters and results for each data fold:
nFolds = 5;
svmResults = cell(nFolds,1);

% For each of 5 data folds:
for i = 1:nFolds
    disp(['Processing data fold ',num2str(i)]);
    
    %%%%%%%%%%%%%%%%%%%
    % Read lists of train, validation, test files for this fold
    [trainFileList,trainLabels] = readLabelList(fullfile(labelDir,['run',num2str(i),'_train.txt']));
    [valFileList,valLabels] = readLabelList(fullfile(labelDir,['run',num2str(i),'_validation.txt']));
    [testFileList,testLabels] = readLabelList(fullfile(labelDir,['run',num2str(i),'_test.txt']));
    
    % Collect pre-computed features for each set
    [trainData,trainLabels] = collectData(trainFileList,trainLabels,nameList,ftrs);
    [valData,valLabels] = collectData(valFileList,valLabels,nameList,ftrs);
    [testData,testLabels] = collectData(testFileList,testLabels,nameList,ftrs);
    
    %%%%%%%%%%%%%%%%%%%
    % Compute distance matrices between datasets (for SVM kernels)
    if( distType == 1 )
        % distType = 1, Mahalanobis distance
        
        % Concatenate files' {mean,cov}'s into ftr matrix
        trainFtr = makeFtrMtx(trainData);
        valFtr = makeFtrMtx(valData);
        testFtr = makeFtrMtx(testData);
        
        dstTrain = mahalDist(trainFtr,trainFtr);
        dstValToTrain = mahalDist(valFtr,trainFtr);
        dstTestToTrain = mahalDist(testFtr,trainFtr);
        
    elseif( distType == 2 )
        % distType = 2, KL divergence
        
        dstTrain = KLdiv(trainData,trainData);
        dstValToTrain = KLdiv(valData,trainData);
        dstTestToTrain = KLdiv(testData,trainData);
    end

    %%%%%%%%%%%%%%%%%%%
    % SVM:
    
    % Number of examples:
    nTrain = size(trainLabels,1);
    nVal = size(valLabels,1);
    nTest = size(testLabels,1);
    
    % Store SVM params and results for each concept: {gamma,C,AP}
    svmResults{i} = zeros(nConcept,3);
    
    %%%%%%%%%%%%%%%%%%%
    % SVMs - 1 per concept:
    for j = 1:nConcept
        disp(['SVM for concept: ',CC{j}]);
        
        % Get label vector for this concept only:
        trainLabelsConcept = trainLabels(:,j);
        valLabelsConcept = valLabels(:,j);
        testLabelsConcept = testLabels(:,j);
        
        %%%%%%%%%%%%%%%%%%%
        % Grid search to select best SVM parameters {gamma,C} (21x10=210 settings)
        % Evaluated on validation data
        gamma = 2.^[-10:10];
        svmC = 10.^[1:10];
        APparams = zeros(length(gamma),length(svmC));
        for kk = 1:length(gamma)
            for mm = 1:length(svmC)
                Gamma = gamma(kk);
                SVMC = svmC(mm);
                
                % Train svm:
                trainKernelMtx = exp(-Gamma * dstTrain);
                svm_model = svmtrain(trainLabelsConcept, [(1:nTrain)',trainKernelMtx], ['-t 4 -c ',num2str(SVMC)]);
                
                % Predict on validation set:
                valKernelMtx = exp(-Gamma * dstValToTrain);
                [tmpLabel,tmpAcc,decisionVal] = svmpredict(valLabelsConcept, [(1:nVal)',valKernelMtx], svm_model);
                
                % Evaluate AP on validation set:
                APparams(kk,mm) = evalAP(decisionVal,valLabelsConcept,svm_model.Label);
            end
        end
        
        % Select params that give best AP over all settings:
        [bestGamma,bestC] = find(APparams == max(max(APparams)));
        bestGamma = bestGamma(1); bestC = bestC(1); % in case tie
        
        %%%%%%%%%%%%%%%%%%%
        % Train/test on test data with final param settings:
        Gamma = gamma(bestGamma);
        SVMC = svmC(bestC);
        trainKernelMtx = exp(-Gamma * dstTrain);
        testKernelMtx = exp(-Gamma * dstTestToTrain);
        svm_model = svmtrain(trainLabelsConcept, [(1:nTrain)',trainKernelMtx], ['-t 4 -c ',num2str(SVMC)]);
        [tmpLabel,tmpAcc,decisionVal] = svmpredict(testLabelsConcept, [(1:nTest)',testKernelMtx], svm_model);
        
        % Get final AP on test data:
        AP = evalAP(decisionVal,testLabelsConcept,svm_model.Label);
        
        % Save SVM data {gamma,C,AP} for each concept
        svmResults{i}(j,1) = Gamma;
        svmResults{i}(j,2) = SVMC;
        svmResults{i}(j,3) = AP;
    end
    
end

%%%%%%%%%%%%%%%%%%%
% Evaluate results:

% Average AP results over 5 folds
finalAP = zeros(nConcept,1);
for i = 1:nFolds
    finalAP = finalAP + svmResults{i}(:,3);
end
finalAP = finalAP / nFolds;

%%%%%%%%%%%%%%%%%%%
% Display results:

disp([' ']);
disp(['%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%']);
disp(['%% Average Precision results: %%']);
disp(['%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%']);
disp([' ']);
for i = 1:nConcept
    disp([CC{i},': ',num2str(finalAP(i),'%2.3f')]);
end
disp(['Mean Average Precision (MAP) over all concepts: ',num2str(mean(finalAP),'%2.3f')]);
disp([' ']);

if( plotResults )

    % Compare against expected AP of guessing
    guessAP = zeros(nConcept,1);
    for i = 1:nFolds
        % Get test labels:
        [testFileList,testLabels] = readLabelList(fullfile(labelDir,['run',num2str(i),'_test.txt']));
        guessAP = guessAP + (sum(testLabels) / size(testLabels,1))';
    end
    guessAP = guessAP / nFolds;
    
    % Sort concept list according to prior (guessAP) and include MAP (mean AP)
    [gAPsort,idxConcept] = sort(guessAP,'ascend');
    CCsort = CC(idxConcept); CCsort = [{'MAP'}; CCsort];
    APsort = finalAP(idxConcept); APsort = [mean(APsort); APsort];
    gAPsort = [mean(gAPsort); gAPsort];
    
    % Compare guess and baseline results
    barh([gAPsort,APsort]);
    set(gca,'yTick',1:length(CCsort),'yTickLabel',CCsort);
    legend('guess','MFCC baseline','Location','Best');
    axis([0 1 0 nConcept+2]);
    xlabel('Average Precision'); ylabel('Concept');
end

%%%%%%%%%%%%%%%%%%%



